#include <at89Lp4052.h>
#include <stdio.h>
#include <stdlib.h>
#include <rtx51tny.h>

#define	SYSTEM		0
#define	KEYBOARD	1
#define	REGULATOR	2

sbit LED1			=	P1^3;										//Volume
sbit LED2			=	P1^4;										//Volume
sbit LED3			=	P1^5;										//Volume
sbit LED4			=	P1^6;										//Volume
sbit LED5			=	P1^7;										//Volume
sbit LED6			=	P3^7;										//LED6 je ako kontrolka

sbit KEY_UP		=	P3^2;
sbit KEY_DOWN	=	P3^3;

typedef enum {ON=0, OFF=1} State;

typedef struct PWMS
{
 unsigned char PWM0,PWM1;
};

struct PWMS VAR = {0x40,0x80};								//Obsahujem predefinovane hodnoty PWM kanalov pri starte

#define	Max 0xFF

#define Kp 1.000															//Proporcionalna konstanta Kp
#define dT 0.005															//Interval vzorkovania pre regulator dT=5ms
#define Ti 0.041															//Integracna konstanta Ti, je potrebne ju prehodnotit pri zmene menica ...
#define Td 0.001															//Derivacna konstanta Td, ovplyvnuje strmost narastu pri zmene reg. odchylky

int			wz = 0;																//Ziadana hodnota otacok
int			ws = 0;																//Skutocna hodnota otacok
float   E0 = 0;									 							//Regulacna odchylka E0 nova
float   E1 = 0;								  							//Regulacna odchylka E1 stara
float   E2 = 0;																//Regulacna odchylka E2 najstarsia
float	  Y0 = 0;								 								//Akcna velicina Y0	nova pre regulator
float   Y1 = (float)Max;											//Akcna velicina Y1 stara pre regulator

#define	KA (Kp*(1.0+dT/(2.0*Ti)+(Td/dT)))    	//Konstanta KA prirastkoveho regulatora
#define KB (Kp*(1.0-dT/(2.0*Ti)+(2.0*Td/dT)))	//Konstanta KB prirastkoveho regulatora
#define	KC (Kp*(Td/dT))												//Konstanta KC prirastkoveho regulatora

void Regulator(void) _task_ REGULATOR
{
 signed char RtxCompletion;
 while(1)
  {
	 switch(RtxCompletion=os_wait(K_IVL+K_SIG,dT,NULL))
	  {
		 case	TMO_EVENT	:	wz=VAR.PWM1;
											ws=100;									//Nacitat zo snimacov inac to nereguluje !!!
											E0=wz-ws;
											Y0=Y1+(KA*E0)-(KB*E1)+(KC*E2);	//Regulacny vztah - vypocet...
											if(Y0<0) Y0=0;
											if(Y0>255) Y0=255;
											RH1=(unsigned char)Y0;
											Y1=Y0;
											E2=E1;
											E1=E0;
											break;
		 case	SIG_EVENT	:	break;
		}
	}
}

void Keyboard(void) _task_ KEYBOARD
{
 signed char RtxCompletion;
 while(1)
  {
	 WDTRST=0x1E;
	 WDTRST=0xE1;
	 switch(RtxCompletion=os_wait(K_TMO+K_SIG,25,NULL))
	  {
		 case	TMO_EVENT	: RtxCompletion=os_send_signal(SYSTEM);
											if(KEY_UP==ON)
											 {
											  switch(VAR.PWM1)
												 {
												  case	0xFF	:	VAR.PWM1=0xFF; break;
													default			:	VAR.PWM1++; break;
												 }
											 }
											if(KEY_DOWN==ON)
											 {
											  switch(VAR.PWM1)
												 {
												  case	0x0ed0	:	VAR.PWM1=0x00; break;
													default			:	VAR.PWM1--; break;
												 }
											 }
											while(KEY_DOWN==ON) RtxCompletion=os_wait(K_TMO,10,NULL); //Vyradit ak chceme pridavat drzanim stlaceneho tlacidla ...
											break;
		 case	SIG_EVENT	:	break;
		}
	}
}

#define	K	(255/5)													// 0 az 255 je 0 az Max rozdelena do 5 LED-iek

void System(void) _task_ SYSTEM
{
 signed char RtxCompletion;
 WDTCON|=(0x80+0x40+0x20);								//Inicializujem Watchdog ...
 TMOD&=~(0x20+0x10);											//9-16bit counter v mode 0, TL1 ma nejake nahodne hodnoty, podla datasheet-u sa nema menit ...
 TMOD&=~0x40;															//C/T=0
 RH1=VAR.PWM1;														//Prednastavim PWM na pociatocne hodnoty, je len 8-bitovy a preto nastavujem len RH1
 TCONB=0x08+0x04;													//PSC10 az PSC12 - nastavenie delicky na 2, default po resete je 0x24
 TCONB|=0x80;															//PWMEN1 povolim PWM1 kanal na P3.5 vyvode procesora
 TR1=1;																		//Spustim casovac T1
 LED1=LED2=LED3=LED4=LED5=ON;							//Rozsvietim vsetky LED na test ..
 P1M0=P3M0=P1M1=P3M1=0x00;								//Inicializujem port P1 a P3 ako bidirectional
 RtxCompletion=os_create_task(KEYBOARD);	//Spustim task obsluhujuci klavesnicu
 RtxCompletion=os_create_task(REGULATOR);	//Spustim task obsluhujuci regulator
 while(1)
  {
	 switch(RtxCompletion=os_wait(K_TMO+K_SIG,15,NULL))
	  {
		 case	TMO_EVENT	: /*RH1=VAR.PWM1;*/ WDTRST=0x1E; WDTRST=0xE1; break;	//Nastavujem WDT a RH1 t.j. PWM1 kanal
		 case	SIG_EVENT	:	switch((unsigned char)(VAR.PWM1/K))
											 {
											  default	:	LED1=LED2=LED3=LED4=LED5=OFF; break;
												case	1	:	LED1=ON; LED2=LED3=LED4=LED5=OFF; break;
												case	2	:	LED1=LED2=ON; LED3=LED4=LED5=OFF; break;
												case	3	:	LED1=LED2=LED3=ON; LED4=LED5=OFF; break;
												case	4	:	LED1=LED2=LED3=LED4=ON; LED5=OFF; break;
												case	5	:	LED1=LED2=LED3=LED4=LED5=ON; break;
											 }
											break;
		}
	}
}